/* File FFINCTRN.C  Create training file for FF network financial */
/*                  modeling/forecasting application */

#ifndef FFINCTRN_C
#define FFINCTRN_C

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <mem.h>
#include <alloc.h>
#include <string.h>

#include "ut.h"
#include "ffin.h"

void initialize_ffin_training_rec (ff_setup_rec ff_setup,
     ffin_training_rec *the_rec);
void create_training_file (ff_setup_rec setup,ffin_training_rec the_rec);

/* Implementation  */

void initialize_ffin_training_rec (ff_setup_rec ff_setup,
     ffin_training_rec *the_rec) {
     short i,j,k;
     the_rec->no_of_data_points = 20;
     the_rec->no_of_input_nodes = ff_setup.no_of_inputs;
     the_rec->no_of_output_nodes = ff_setup.no_of_outputs;
     for (i = 0;i<MAX_NO_OF_INPUT_NODES;i++) {
	  strcpy(the_rec->input_node_data[i].file_name,"");
	  the_rec->input_node_data[i].start.year = 1962;
	  the_rec->input_node_data[i].start.qtr = 1;
	  }  /*  i  */
     for (i = 0;i<MAX_NO_OF_OUTPUT_NODES;i++) {
	  strcpy(the_rec->output_node_data[i].file_name,"");
	  the_rec->output_node_data[i].start.year = 1962;
	  the_rec->output_node_data[i].start.qtr = 2;
	  }  /*  i  */
     return;
     }  /*  end proc  */

short read_file_data (path_str data_name,qtr_date start,
      int no_of_items, tdata_arr the_data) {
      FILE *infile;
      int in_year,in_qtr,item = 0;
      unsigned short start_flag = 0;
      float in_data;
      #define RECORD_LEN 80
      char file_record[RECORD_LEN];
      if (!open_input_text_file (&infile,data_name)) return 0;
      cprintf ("\r\n\nReading %s...",data_name);
      /* Read past header record:  */
      fgets (file_record,RECORD_LEN,infile);
      while ((item < no_of_items) &&
	    (fgets (file_record,RECORD_LEN,infile) != NULL)) /* eof */ {
	   if (sscanf (file_record,"%d %d %f\n",&in_year,&in_qtr,&in_data)
		!= 3) {
	       cprintf ("\r\nProblem reading data file %s",data_name);
	       cputs ("\r\n\nPress key to cancel: ");
	       getch();
	       fclose (infile);
	       return 0;
	       }
	   /*  Check year and quarter for starting point.  Once starting */
	   /*  point has been reached, collect next no_of_items points   */
	   /*  (regardless of possible gaps in data).  */
	   if (!start_flag)
	      if ((in_year >= start.year) &&
		 (in_qtr >= start.qtr)) start_flag = 1;
	   if (start_flag)
	      if (item < no_of_items) {
		 the_data[item] = in_data;
		 item++;
		 }
	}  /*  end while  */
	if (item < no_of_items) {
	    cprintf ("\r\nInsufficient data in file %s "
		 "to process %d items.",data_name,
		     no_of_items);
	    cputs ("\r\n\nPress any key to cancel: ");
	    getch();
	    fclose (infile);
	    return 0;
	    }
	fclose (infile);
	return 1;
	}  /* end proc */

short read_data (ffin_training_rec the_rec,tdata_arr *input_data,
	   tdata_arr *output_data) {
      int i,no_of_test_items = the_rec.no_of_data_points +
	    the_rec.no_of_look_ahead_points;
      for (i = 0;i<the_rec.no_of_input_nodes;i++)
	  if (!read_file_data (the_rec.input_node_data[i].file_name,
	       the_rec.input_node_data[i].start,no_of_test_items,
	       input_data[i])) return 0;
      for (i = 0;i<the_rec.no_of_output_nodes;i++)
	  if (!read_file_data (the_rec.output_node_data[i].file_name,
	       the_rec.output_node_data[i].start,no_of_test_items,
	       output_data[i])) return 0;
      return 1;
      }  /* end proc */

short write_output_file (path_str the_file_name, ffin_training_rec the_rec,
      float max_in[],float min_in[],float max_out[],float min_out[],
      tdata_arr *input_data,tdata_arr *output_data,
      int no_of_items) {
      FILE *outfile;
      int i,j,line;
      clrscr();
      if (!open_output_text_file (&outfile,the_file_name)) return 0;
      line = wherey() + 1;
      fprintf (outfile,"%3d  %2d\n",the_rec.no_of_input_nodes,
	  the_rec.no_of_output_nodes);
      fprintf (outfile,"minimum\n");
      for (i = 0;i<the_rec.no_of_input_nodes;i++)
	  fprintf (outfile,"%7.2f\n",min_in[i]);
      for (i = 0;i<the_rec.no_of_output_nodes;i++)
	  fprintf (outfile,"%7.2f\n",min_out[i]);
      fprintf (outfile,"maximum\n");
      for (i = 0;i<the_rec.no_of_input_nodes;i++)
	  fprintf (outfile,"%7.2f\n",max_in[i]);
      for (i = 0;i<the_rec.no_of_output_nodes;i++)
	  fprintf (outfile,"%7.2f\n",max_out[i]);
      fprintf (outfile,"training data\n");

	for (i = 0;i<no_of_items;i++) {
	    for (j = 0;j<the_rec.no_of_input_nodes;j++)
		fprintf (outfile,"%7.2f\n",input_data[j][i]);
	    for (j = 0;j<the_rec.no_of_output_nodes;j++)
		fprintf (outfile,"%7.2f\n",output_data[j][i]);
	    gotoxy (1,line);
	    cputs ("                    ");
	    gotoxy (1,line);
	    cprintf ("%3d of %3d items done.",i + 1,no_of_items);
	    }  /* i */
	fclose (outfile);
	return 1;
	}  /* end fn */

void create_training_file (ff_setup_rec setup,ffin_training_rec the_rec) {
     tdata_arr *input_nodes_data = NULL, *output_nodes_data = NULL;
     float max_in[MAX_NO_OF_INPUT_NODES],
	   min_in[MAX_NO_OF_INPUT_NODES],
	   max_out[MAX_NO_OF_OUTPUT_NODES],
	   min_out[MAX_NO_OF_OUTPUT_NODES];
     float range;
     int i,j,no_of_training_items = the_rec.no_of_data_points,
	     no_of_test_items = the_rec.no_of_data_points +
				   the_rec.no_of_look_ahead_points;
     short line;

     clrscr();
     textcolor (YELLOW);
     gotoxy (1,1);
     cputs ("Neural Net Financial Forecasting: Create Training"
	" and Test Files");
     gotoxy (1,3);
     /* Check for blank file names  */
     if (setup.files.training_file_name[0] == '\0') {
	cputs ("\r\nTraining file name is blank.\r\n\nPress key to cancel: ");
	getch();
	return;
	}
     if (setup.files.test_file_name[0] == '\0') {
	cputs ("\r\nTest file name is blank.\r\n\nPress key to cancel: ");
	getch();
	return;
	}
     for (i = 1;i<=the_rec.no_of_input_nodes;i++)
	 if (the_rec.input_node_data[i-1].file_name[0] == '\0') {
	 cprintf ("\r\nInput Node #%d file name is blank."
	     "\r\n\nPress key to cancel: ",i);
	 getch();
	 return;
	 }
     for (i = 1;i<=the_rec.no_of_output_nodes;i++)
	 if (the_rec.output_node_data[i-1].file_name[0] == '\0') {
	 cprintf ("\r\nOutput Node #%d file name is blank."
	     "\r\n\nPress key to cancel: ",i);
	 getch();
	 return;
	 }
     cprintf ("\r\n\nOutput Training File: %s",setup.files.training_file_name);
     cprintf ("\r\nOutput Test File: %s",setup.files.test_file_name);
     cprintf ("\r\n\nNumber of training file points: %d",
	 no_of_training_items);
     cprintf ("\r\nNumber of test file points: %d",
	 no_of_test_items);
     cprintf ("\r\n\nNumber of network input nodes: %d",
	 the_rec.no_of_input_nodes);
     for (i = 1;i<=the_rec.no_of_input_nodes;i++) {
	 cprintf ("\r\nInput Node #%d File Name: %s",i,
		 the_rec.input_node_data[i-1].file_name);
	 cprintf ("\r\nStart Year: %d, Qtr: %d",
	     the_rec.input_node_data[i-1].start.year,
	     the_rec.input_node_data[i-1].start.qtr);
	 }  /*  i  */
     cprintf ("\r\n\nNumber of network output nodes: %d",
	 the_rec.no_of_output_nodes);
     for (i = 1;i<=the_rec.no_of_output_nodes;i++) {
	 cprintf ("\r\nOutput Node #%d File Name: %s",i,
		 the_rec.output_node_data[i-1].file_name);
	 cprintf ("\r\nStart Year: %d, Qtr: %d",
	     the_rec.output_node_data[i-1].start.year,
	     the_rec.output_node_data[i-1].start.qtr);
	 }  /*  i  */
     cputs ("\r\n\nPress any key to start (esc to cancel): ");
     if (getch() == ESC_KEY) return;

     input_nodes_data = (tdata_arr *)malloc(the_rec.no_of_input_nodes *
	   sizeof(tdata_arr));
     output_nodes_data = (tdata_arr *)malloc(the_rec.no_of_output_nodes *
	   sizeof(tdata_arr));

     /* Initialize data arrays  */
     for (i=0;i<the_rec.no_of_input_nodes;i++)
	 for (j = 0;j<MAX_NO_OF_INPUT_ITEMS;j++)
	     input_nodes_data[i][j] = 0.0;
     for (i=0;i<the_rec.no_of_output_nodes;i++)
	 for (j = 0;j<MAX_NO_OF_INPUT_ITEMS;j++)
	     output_nodes_data[i][j] = 0.0;

     clrscr();
     if (!read_data (the_rec,input_nodes_data,output_nodes_data))
	goto exit_proc;


     /*  Determine max-min based on training points  */
     /*  (No fair using test points for max-min determination)  */
     for (i=0;i<the_rec.no_of_input_nodes;i++) {
	 max_in[i] = input_nodes_data[i][0];
	 min_in[i] = input_nodes_data[i][0];
	 for (j=0;j<no_of_training_items;j++) {
	     if (input_nodes_data[i][j] > max_in[i])
		max_in[i] = input_nodes_data[i][j];
	     if (input_nodes_data[i][j] < min_in[i])
		min_in[i] = input_nodes_data[i][j];
	     }  /*  j  */
	 /*  Allow for expansion  */
	 range = max_in[i] - min_in[i];
	 max_in[i] += 0.1 * range;
	 min_in[i] -= 0.1 * range;
	 }  /*  i  */
     for (i=0;i<the_rec.no_of_output_nodes;i++) {
	 max_out[i] = output_nodes_data[i][0];
	 min_out[i] = output_nodes_data[i][0];
	 for (j=0;j<no_of_training_items;j++) {
	     if (output_nodes_data[i][j] > max_out[i])
		max_out[i] = output_nodes_data[i][j];
	     if (output_nodes_data[i][j] < min_out[i])
		min_out[i] = output_nodes_data[i][j];
	     }  /*  j  */
	 /*  Allow for expansion  */
	 range = max_out[i] - min_out[i];
	 max_out[i] += 0.1 * range;
	 min_out[i] -= 0.1 * range;
	 }  /*  i  */

     if (!write_output_file (setup.files.training_file_name,the_rec,
	max_in,min_in,max_out,min_out,input_nodes_data,output_nodes_data,
	no_of_training_items)) goto exit_proc;
     if (!write_output_file (setup.files.test_file_name,the_rec,
	max_in,min_in,max_out,min_out,input_nodes_data,output_nodes_data,
	no_of_test_items)) goto exit_proc;

     line = wherey() + 1;
     gotoxy (1,line + 2);
     cputs ("Done.");
     gotoxy (1,line + 2);
     cputs ("Press key to clear: ");
     getch();

exit_proc:
     if (input_nodes_data) free (input_nodes_data);
     if (output_nodes_data) free (output_nodes_data);
     textmode (LASTMODE);
     return;
     }  /* end proc */

#endif
