/* File SLFGRAPH.C Self Organizing neural network with graphics */

#ifndef SLFGRAPH_C
#define SLFGRAPH_C

#include <stdlib.h>
#include <conio.h>
#include <math.h>  /* for fabs */
#include "ut.h"
#include "utgr.h"
#include "slforg.h"

#define no_of_inputs 2

int gr_page = 0;

void initialize_weights (float ***w,int n_inp, int l_rows, int l_cols)
   {
   int i,j,k;
   float r;
   srand(1);
   for (k = 1;k <= n_inp;k++) {
       for (i = 1;i <= l_rows;i++) {
	   for (j = 1;j <= l_cols;j++) {
		r = 2*frand() - 1.0;  /*Random number between -1 and 1*/
		  w [i] [j] [k] = 0.1 * r; /*Choose small initial weight values*/
		  } /*j*/
	      } /*i*/
	  } /*k*/
      }  /*procedure*/

float gaussian (const float x,const float y) {
   return exp (-x*x - y*y);
   }

void henon_dynam_sys (float a,float b,float x_in,float y_in,
   float *x_out,float *y_out) {
   *x_out = 1.0 + y_in - a * x_in * x_in;
   *y_out = b * x_in;
   }

void write_graph_screen (self_org_setup_rec s,
   float ***w, int l_rows, int l_cols,short color,unsigned long iters)
   {
   int i,j;
   int col,row;
   int col1,row1,col2,row2;
   int active_page,visual_page;
   gr_page++;
   visual_page = gr_page % 2;  /* % = mod operator */
   active_page = (gr_page + 1) % 2;
   setvisualpage (visual_page);
   setactivepage (active_page);
   cleardevice();
   draw_border(s.gr_rec);
   setcolor (color);
   for (i = 1;i <= l_rows - 1;i++) {
       for (j = 1;j <= l_cols - 1;j++) {
	   xy_to_screen_col_row (w[i][j][1], w [i][j][2],&col,&row);
	   xy_to_screen_col_row (w[i][j+1][1], w[i][j+1][2],&col1,&row1);
	   xy_to_screen_col_row (w[i+1][j][1], w[i+1][j][2],&col2,&row2);
	   line (col,row,col1,row1);
	   line (col,row,col2,row2);
	   } /*j*/
      /*Do the last point in this row*/
      xy_to_screen_col_row (w[i][l_cols][1],
		       w[i][l_cols][2],&col,&row);
      xy_to_screen_col_row (w[i+1][l_cols][1],
		       w[i+1][l_cols][2],&col1,&row1);
      line (col,row,col1,row1);
      } /*i*/
      /*Do the last row*/
      for (j = 1;j <=l_cols - 1;j++) {
	  xy_to_screen_col_row (w[l_rows][j][1],
		     w[l_rows][j][2],&col,&row);
	  xy_to_screen_col_row (w[l_rows][j+1][1],
		     w[l_rows][j+1][2],&col1,&row1);
	  line (col,row,col1,row1);
	  } /*j*/
       graph_iter (s.gr_rec,iters);
       setvisualpage(active_page);
       }

void self_org_learning (self_org_setup_rec s)
   {
   float xx[no_of_inputs],*x;
   float ***w;
   int i,j,k,min_i,min_j;
   const prelim_iters = 100;
   const float too_big = 1e+4;
   float x_old = 0.0,y_old = 0.0,x_new = 0.0,y_new = 0.0;
   float xmin = 0.0,ymin = 0.0,xmax = 0.0,ymax = 0.0;
   float min_dist;
   int nbhd = s.starting_nbhd;
   float factor, alpha = 1.0/s.starting_nbhd;
   unsigned long iters = 0;
   float stepsize = s.starting_stepsize;
   float sum;
   int i_lower_limit,i_upper_limit,
       j_lower_limit,j_upper_limit;
   x = xx - 1;  /* So x has subscript range 1..no_of_inputs */

    clrscr();
    textcolor (YELLOW);
    gotoxy (1,5);
    cprintf ("Press key to start (esc to cancel): \r\n");
    if (getch() == ESC_KEY) return;
    if (s.the_sys == Henon_sys) {
    cprintf ("Determining max-min values...\r\n");
    for (i = 1;i < prelim_iters; i++){
	henon_dynam_sys (s.a,s.b,x_old,y_old,&x_new,&y_new);
	if ((fabs (x_new) > too_big) || (fabs(y_new) > too_big)) {
	   gotoxy (1,7);
	   cprintf ("This system diverges.  Press key to cancel: ");
	   getch();
	   return;
	   }
    if (x_new < xmin) xmin = x_new;
    if (x_new > xmax) xmax = x_new;
    if (y_new < ymin) ymin = y_new;
    if (y_new > ymax) ymax = y_new;
    x_old = x_new;
    y_old = y_new;
    }  /* end for loop */

    if (s.gr_rec.scale_window_from_data != 0) {
	s.gr_rec.x_min = xmin;
	s.gr_rec.x_max = xmax;
	s.gr_rec.y_min = ymin;
	s.gr_rec.y_max = ymax;
	}
    }
    if (!graph_setup (s.gr_rec)) return;
    draw_border(s.gr_rec);
    w = matrix3d (1,s.lattice_rows,1,s.lattice_cols,1,no_of_inputs);
    initialize_weights(w,no_of_inputs,s.lattice_rows,s.lattice_cols);

    while (!kbhit()) {
	iters++;

	switch (s.the_sys) {
	case unif_dist:
	     /*Random input between -1 and 1*/
	     for (k = 1; k <= no_of_inputs; k++)
		  x [k] = 2*frand() - 1.0;
	     break;
	case Henon_sys:
	     henon_dynam_sys (s.a,s.b,x_old,y_old,&x[1],&x[2]);
	     x_old = x[1];
	     y_old = x[2];
	     break;
	}
	/*Initialize minimum distance*/
	sum = 0;
	for (k = 1;k <= no_of_inputs; k++)
	    sum += (x[k] - w[1][1][k])*(x[k] - w[1][1][k]);
	min_i = 1;
	min_j = 1;
	min_dist = sum;

	/*Find minimum distance over all weights*/
	for (i = 1;i <= s.lattice_rows; i++)
	    for (j = 1; j <= s.lattice_cols; j++) {
		sum = 0;
		for (k = 1; k <= no_of_inputs; k++)
		    sum += (x[k] - w[i][j][k])*(x[k] - w[i][j][k]);
		if (sum < min_dist) {
		   min_i = i;
		   min_j = j;
		   min_dist = sum;
		   }  /*if*/
		} /*i,j*/

	/*Update weights in neighborhood around minimum*/

	/*First, make sure neighborhood doesn't exceed array bounds*/

	if ((min_i - nbhd) < 1) i_lower_limit = 1;
	   else i_lower_limit = min_i - nbhd;
	if ((min_j - nbhd) < 1) j_lower_limit = 1;
	   else j_lower_limit = min_j - nbhd;
	if ((min_i + nbhd) > s.lattice_rows)
	   i_upper_limit = s.lattice_rows;
	   else
	   i_upper_limit = min_i + nbhd;
	if ((min_j + nbhd) > s.lattice_rows)
	   j_upper_limit = s.lattice_rows;
	   else
	   j_upper_limit = min_j + nbhd;

	for (i = i_lower_limit;i <= i_upper_limit; i++)
	    for (j = j_lower_limit;j <= j_upper_limit; j++) {
		factor = gaussian (alpha*(i-min_i),alpha*(j-min_j));
		for (k = 1; k <= no_of_inputs; k++)
		    w [i][j][k] += factor*stepsize*(x [k] - w [i][j][k]);
		    }

       /*Display weight values on screen, every freq'th iteration*/

       if ((iters == 1) || ((iters %  s.freq) == 0))
	  write_graph_screen(s,w,s.lattice_rows,s.lattice_cols,s.color,
	      iters);

       /*Update stepsize and nbhd size*/

       if (iters % s.iter_interval == 0) {
	  stepsize *= 0.8;
	  nbhd -= 1;
	  if (nbhd < 1) nbhd = 1;
	  }
       }  /* while not keypressed...*/
       getch();
       outtextxy (gr_iter_col,gr_done_row,"Press key ");
       outtextxy (gr_iter_col,gr_done_row + textheight("P") + 2,"to clear.");
       getch();
       closegraph();
       free_matrix3d(w,1,1,1);
       textmode (LASTMODE);
       return;
       }
#endif



