//  File UISETUP.CPP General User Interface Setup

#ifndef UISETUP_CPP
#define UISETUP_CPP

#include <fcntl.h>
#include <io.h>
#include <sys\stat.h>

extern "C" {           // extern "C" used to avoid name mangling
   #include "ut.h"
   #include "uiwiley.c"    // Wiley startup message
   }

#define Uses_TObject
#define Uses_TEventQueue
#define Uses_TEvent
#define Uses_TProgram
#define Uses_TApplication
#define Uses_TKeys
#define Uses_TRect
#define Uses_TMenuBar
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TStatusLine
#define Uses_TStatusItem
#define Uses_TStatusDef
#define Uses_TDeskTop
#define Uses_TView
#define Uses_TWindow
#define Uses_TFrame
#define Uses_TScroller
#define Uses_TScrollBar
#define Uses_TFileDialog

#include "uilstdlg.cpp"
#include "uiclrobj.cpp"

const
    cmNull = cmError,  // Not handled by any class
    cmAbout = 200,
    cmSaveWtsAs = 202,
    cmRun = 203,
    cmSetup = 204,
    cmPrintSetupRec = 205,
    cmSaveSetupRecAs = 206,
    cmSaveSetupRec = 207,
    cmOpenSetupRec = 208,
    cmTrain = 209,
    cmTrainingFile = 210,
    cmPrintTrainRec = 211,
    cmSetupTrainRec = 212,
    cmSaveTrainRecAs = 213,
    cmSaveTrainRec = 214,
    cmOpenTrainRec = 215,
    cmSetupData = 216,
    cmEditData = 217,
    cmNewData = 218,
    cmSpecial = 219;

TMenuItem *about_submenu,
	  *file_submenu,
	  *run_submenu,
	  *setup_submenu,
	  *train_submenu,
	  *data_submenu,
	  *special_submenu,
	  *about_item,
	  *save_wts_as_item,
	  *quit_item,
	  *run_item,
	  *open_setup_item,
	  *save_setup_item,
	  *save_setup_as_item,
	  *print_setup_item,
	  *setup_item,
	  *open_trn_item,
	  *save_trn_item,
	  *save_trn_as_item,
	  *print_trn_item,
	  *setup_trn_item,
	  *create_trn_item,
	  *train_item,
	  *data_setup_item,
	  *data_edit_item,
	  *new_data_item,
	  *special_item;

// Borland doesn't supply an overloaded '+' for two TMenuItems,
// so we need to do our own:
TMenuItem& operator + (TMenuItem& i1, TMenuItem& i2)
{
    TMenuItem *item = &i1;
    while( item->next != 0 )
	item = item->next;
    item->next = &i2;
    return i1;
}

void write_record (char *the_file_name,void *the_rec,int the_size);
void read_record (char *the_file_name,void *the_rec,int the_size);

class TSetApp : public TApplication
  {
  public:
     tlist_box_data system_setup_data,training_setup_data,data_setup_data;
     TSetApp();
     static TStatusLine *initStatusLine (TRect r );
     static void init_submenus();
     static TMenuBar *initMenuBar (TRect r );
     virtual void handleEvent (TEvent& event);
     virtual void about_dialog();
     virtual void copy_wts_file (const char *old_name,const char *new_name);
     virtual void print_parameters (const char *the_title,
	tlist_box_data the_data_rec);
     virtual char *save_setup (char *the_file_name);
     virtual char *read_setup (char *the_file_name);
     virtual char *save_training_setup (char *the_file_name);
     virtual char *read_training_setup (char *the_file_name);
     virtual void setup();
     virtual void training_setup();
     virtual void data_file_setup();
     static void destroy (TSetApp *ob);
  };

TSetApp::TSetApp() :
   TProgInit (&TSetApp::initStatusLine,
	      &TSetApp::initMenuBar,
	      &TSetApp::initDeskTop
	      )
{
   gdesk_top = TSetApp::deskTop;
   gthe_app = this;
   system_setup_data.item_collection = new TDataCollection
      (MAX_NO_OF_SETUP_ITEMS,NO_GROWTH);
   training_setup_data.item_collection = new TDataCollection
      (MAX_NO_OF_SETUP_ITEMS,NO_GROWTH);
   data_setup_data.item_collection = new TDataCollection
      (MAX_NO_OF_SETUP_ITEMS,NO_GROWTH);
   _setcursortype(_NOCURSOR);
   TMouse::show();
}

void TSetApp::destroy (TSetApp *ob) {
     TCollection::destroy (ob->system_setup_data.item_collection);
     TCollection::destroy (ob->training_setup_data.item_collection);
     TCollection::destroy (ob->data_setup_data.item_collection);
     TApplication::destroy (ob);
     TMouse::hide();
     _setcursortype(_NORMALCURSOR);
     return;
     }

void TSetApp::init_submenus ()
 {
    // '+' is overloaded TMenuItem operator defined above.
    // Special menu items:
    special_item = new TMenuItem ("~S~pecial Item",cmSpecial,kbNoKey,
       hcNoContext);
    special_submenu = new TMenuItem ("S~p~ecial",kbAltP,
       new TMenu(*special_item),hcNoContext,0);
    // Data menu items:
    new_data_item = new TMenuItem ("~N~ew Data",cmNewData,kbNoKey);
    data_edit_item = new TMenuItem ("~E~dit Data File",cmEditData,kbNoKey,
	 hcNoContext,0,new_data_item);
    data_setup_item = new TMenuItem ("~S~etup Data...",cmSetupData,
	 kbNoKey,hcNoContext,0,&(newLine() + *data_edit_item));
    data_submenu = new TMenuItem ("~D~ata",kbAltD,new TMenu(*data_setup_item),
	 hcNoContext,special_submenu);
    // Train menu items:
    train_item = new TMenuItem ("~T~rain Network",cmTrain,kbCtrlF9,
	 hcNoContext,"Ctrl-F9",0);
    create_trn_item = new TMenuItem ("~C~reate Training File",cmTrainingFile,
	 kbNoKey,hcNoContext,0,&(newLine() + *train_item));
    setup_trn_item = new TMenuItem ("~S~etup Training Rec...",cmSetupTrainRec,
	 kbNoKey,hcNoContext,0,&(newLine() + *create_trn_item));
    print_trn_item = new TMenuItem ("~P~rint Training Rec",
	  cmPrintTrainRec,kbNoKey,hcNoContext,0,&(newLine() + *setup_trn_item));
    save_trn_as_item = new TMenuItem ("S~a~ve Training Rec as...",
	  cmSaveTrainRecAs,kbNoKey,hcNoContext,0,print_trn_item);
    save_trn_item = new TMenuItem ("Sa~v~e Training Rec",cmSaveTrainRec,
	  kbNoKey,hcNoContext,0,save_trn_as_item);
    open_trn_item = new TMenuItem ("~O~pen Training Rec...",cmOpenTrainRec,
	  kbNoKey,hcNoContext,0,save_trn_item);
    train_submenu = new TMenuItem ("~T~rain",kbAltT,new TMenu(*open_trn_item),
	  hcNoContext,data_submenu);
    // Setup menu items:
    setup_item = new TMenuItem ("~S~etup System...",cmSetup,kbF3,hcNoContext,
	 "F3",0);
    print_setup_item = new TMenuItem ("~P~rint Setup Rec",
	  cmPrintSetupRec,kbNoKey,hcNoContext,0,&(newLine() + *setup_item));
    save_setup_as_item = new TMenuItem ("S~a~ve Setup Rec as...",
	  cmSaveSetupRecAs,kbNoKey,hcNoContext,0,
	  print_setup_item);
    save_setup_item = new TMenuItem ("Sa~v~e Setup Rec",cmSaveSetupRec,
	  kbNoKey,hcNoContext,0,save_setup_as_item);
    open_setup_item = new TMenuItem ("~O~pen Setup Rec...",cmOpenSetupRec,
	  kbNoKey,hcNoContext,0,save_setup_item);
    setup_submenu = new TMenuItem ("~S~etup",kbAltS,new TMenu(*open_setup_item),
	  hcNoContext,train_submenu);
    setup_submenu->subMenu->deflt = setup_item; //Make setup the default item
    // Run menu items:
    run_item = new TMenuItem ("~R~un Network",cmRun,kbCtrlF4,hcNoContext,
	  "Ctrl-F4",0);
    run_submenu = new TMenuItem ("~R~un",kbAltR,new TMenu(*run_item),
	  hcNoContext,setup_submenu);
    // File menu items:
    quit_item = new TMenuItem ("~Q~uit",cmQuit,kbAltQ,hcNoContext,
	  "Alt-Q",0);
    save_wts_as_item = new TMenuItem ("S~a~ve Weights as...",cmSaveWtsAs,
	  kbNoKey,hcNoContext,0,&(newLine() + *quit_item));
    file_submenu = new TMenuItem ("~F~ile",kbAltF,new TMenu(*save_wts_as_item),
	  hcNoContext,run_submenu);
    file_submenu->subMenu->deflt = quit_item;
    // About menu items:
    about_item = new TMenuItem ("~A~bout...",cmAbout,kbNoKey,hcNoContext,
	  0,0);
    about_submenu = new TMenuItem ("~" "\xF0" "~",kbAltA,
	   new TMenu(*about_item),hcNoContext,file_submenu);
    }

TMenuBar *TSetApp::initMenuBar ( TRect r )
{
   r.b.y = r.a.y + 1;  //Set bottom line 1 line below top line
   init_submenus();
   return new TMenuBar (r,new TMenu(*about_submenu));
 }

TStatusLine *TSetApp::initStatusLine ( TRect r)
 {
    r.a.y = r.b.y - 1;
    return new TStatusLine( r,
    *new TStatusDef( 0, 0xFFFF ) +
       *new TStatusItem( "~Alt-Q~ Quit",kbAltQ,cmQuit ) +
       *new TStatusItem( "~F10~ Menu", kbF10, cmMenu ) +
       *new TStatusItem( "~F3~ Setup System",kbF3,cmSetup ) +
       *new TStatusItem( "~Ctrl-F9~ Train",kbCtrlF9,cmTrain) +
       *new TStatusItem( "~Ctrl-F4~ Run",kbCtrlF4,cmRun )
       );
 }

 void TSetApp::handleEvent (TEvent& event)
 {
    path_str file_name;
    TApplication::handleEvent(event);
    if (event.what == evCommand) {
       switch (event.message.command) {
	  case cmAbout:
	     about_dialog();
	     break;
	  case cmSaveWtsAs:
	     file_name[0] = '\0';
	     get_file_name_dlg ("*.WTS","WEIGHTS FILE",file_name);
	     if (file_name[0] != '\0')
	      copy_wts_file ((char *)&ui_files->out_wts_file_name,file_name);
	     else
	      messageBox ("Invalid WEIGHTS file name.",mfError | mfOKButton);
	     break;
	  case cmRun:
	     quit_flag = run_net;
	     TApplication::endModal(cmQuit); // End Application
	     break;
	  case cmOpenSetupRec:
	     file_name[0] = '\0';
	     get_file_name_dlg ("*.SET","SETUP RECORD",file_name);
	     if (file_name[0] != '\0') {
		read_setup (file_name);
		setup();
		}
	     else
	      messageBox ("Invalid SETUP file name.",mfError | mfOKButton);
	     break;
	  case cmSaveSetupRec:
	     if (ui_files->setup_file_name[0] == '\0')
		get_file_name_dlg ("*.SET","SETUP RECORD",
		   (char *)&(ui_files->setup_file_name));
	     if (ui_files->setup_file_name[0] != '\0')
		save_setup ((char *)&(ui_files->setup_file_name));
	     else
	      messageBox ("Invalid SETUP file name.",mfError | mfOKButton);
	     break;
	  case cmSaveSetupRecAs:
	     file_name[0] = '\0';
	     get_file_name_dlg ("*.SET","SETUP RECORD",file_name);
	     if (file_name[0] != '\0') {
		save_setup (file_name);
		strcpy(ui_files->setup_file_name,file_name);
		}
	     else
	      messageBox ("Invalid SETUP file name.",mfError | mfOKButton);
	     break;
	  case cmSetup:
	     setup();
	     break;
	  case cmPrintSetupRec:
	     print_parameters (system_name,system_setup_data);
	     break;
	  case cmOpenTrainRec:
	     file_name[0] = '\0';
	     get_file_name_dlg ("*.TRC","TRAINING SETUP",file_name);
	     if (file_name[0] != '\0') {
		read_training_setup (file_name);
		training_setup();
		}
	     else
	      messageBox ("Invalid TRAINING SETUP file name.",mfError | mfOKButton);
	     break;
	  case cmSaveTrainRec:
	     if (ui_files->training_setup_file_name[0] == '\0')
		get_file_name_dlg ("*.TRC","TRAINING SETUP",
		   (char *)&(ui_files->training_setup_file_name));
	     if (ui_files->training_setup_file_name[0] != '0')
		save_training_setup
		    ((char *)&(ui_files->training_setup_file_name));
	     else
	      messageBox ("Invalid TRAINING SETUP file name.",mfError | mfOKButton);
	     break;
	  case cmSaveTrainRecAs:
	     file_name[0] = '\0';
	     get_file_name_dlg ("*.TRC","TRAINING SETUP",file_name);
	     if (file_name[0] != '\0') {
		save_training_setup (file_name);
		strcpy(ui_files->training_setup_file_name,file_name);
		}
	     else
	      messageBox ("Invalid TRAINING SETUP file name.",mfError | mfOKButton);
	     break;
	  case cmSetupTrainRec:
	     training_setup();
	     break;
	  case cmPrintTrainRec:
	     print_parameters ("Training",training_setup_data);
	     break;
	  case cmTrainingFile:
	     if (ui_files->training_file_name[0] == '\0')
		get_file_name_dlg ("*.TRN","TRAINING FILE",
		   (char *)&(ui_files->training_file_name));
	     if (ui_files->training_file_name[0] != '\0') {
		quit_flag = training_data;
		TApplication::endModal(cmQuit); // End Application
		}
	     else
	      messageBox ("Invalid TRAINING file name.",mfError | mfOKButton);
	     break;
	  case cmTrain:
	     quit_flag = train_net;
	     TApplication::endModal(cmQuit); // End Application
	     break;
	  case cmSetupData:
	     data_file_setup();
	     break;
	  case cmEditData:
	     if (ui_files->data_file_name[0] == '\0')
		get_file_name_dlg ("*.DAT","DATA FILE",
		   (char *)&(ui_files->data_file_name));
	     if (ui_files->new_data_file_name[0] == '\0')
		get_file_name_dlg ("*.DAT","NEW DATA FILE",
		   (char *)&(ui_files->new_data_file_name));
	     if ((ui_files->data_file_name[0] != '\0') &&
		(ui_files->new_data_file_name[0] != '\0')) {
		quit_flag = edit_data;
		TApplication::endModal(cmQuit); // End Application
		}
	     else
	      messageBox ("Invalid DATA file name.",mfError | mfOKButton);
	     break;
	  case cmNewData:
	     if (ui_files->new_data_file_name[0] == '\0')
		get_file_name_dlg ("*.DAT","NEW DATA FILE",
		   (char *)&(ui_files->new_data_file_name));
	     if (ui_files->new_data_file_name[0] != '\0') {
		quit_flag = new_data;
		TApplication::endModal(cmQuit); // End Application
		}
	     else
	      messageBox ("Invalid DATA file name.",mfError | mfOKButton);
	     break;
	  case cmSpecial:
	     quit_flag = special;
	     TApplication::endModal(cmQuit);
	     break;
	  default:
	     return;
	  }
	  clearEvent(event);
	  }
     }

void TSetApp::about_dialog()
   {
   const x_start = 22,
	 box_width = 36,
	 half_width = box_width/2,
	 y_start = 3,
	 box_height = 17;
   TRect r = TRect (x_start,y_start,x_start + box_width,y_start +
      box_height);
   TDialog *dialog = new TDialog(r,"ABOUT");
   int line = 2;
   insert_static_text (dialog,line,"Neural Network and Fuzzy Logic");
   line += 1;
   insert_static_text (dialog,line,"Applications in C/C++");
   line += 2;
   insert_static_text (dialog,line,"by");
   line += 2;
   insert_static_text (dialog,line,"Stephen T. Welstead");
   line += 2;
   insert_static_text (dialog,line,"System Name:");
   line += 2;
   insert_static_text (dialog,line,system_name);
   r = TRect (half_width - 8,box_height - 4,half_width + 8,
      box_height - 2);
   dialog->insert(new TButton(r,"O~K~ (Enter)",cmOK,bfDefault));
   validView(dialog);
   if (dialog != 0) gdesk_top->execView(dialog);
   }

void TSetApp::copy_wts_file (const char *old_name,const char *new_name)
   {
   switch (fcopy (old_name,new_name)) {
     case 0: messageBox ("Weights Saved",mfInformation |
			mfOKButton);
	     break;
     case -1: messageBox ("Out Wts File Name not Valid",mfError | mfOKButton);
	      break;
     case -2: messageBox ("New Wts File Name not Valid",mfError | mfOKButton);
	      break;
   }
   return;
   }

char *TSetApp::save_setup (char *the_file_name)
   {
// Override in descendant classes
   return the_file_name;
   }

char *TSetApp::read_setup (char *the_file_name)
   {
// Override in descendant classes
   return the_file_name;
   }

char *TSetApp::save_training_setup (char *the_file_name)
   {
// Override in descendant classes
   return the_file_name;
   }

char *TSetApp::read_training_setup (char *the_file_name)
   {
// Override in descendant classes
   return the_file_name;
   }

void TSetApp::setup()
   {
// Override in descendant classes
   return;
   }

void TSetApp::training_setup()
   {
// Override in descendant classes
   return;
   }

void TSetApp::data_file_setup()
   {
// Override in descendant classes
   return;
   }

void TSetApp::print_parameters (const char *the_title,
	tlist_box_data the_data_rec)
   {
   const int max_str_len = 80;
   char the_string[max_str_len];
   fprintf (stdprn,"\n%s Parameters\n\n",the_title);
   for (int i = 0;i<the_data_rec.item_collection->getCount();i++)
       if (((tcolor_data_obj *) (the_data_rec.item_collection)->at(i))
	 ->enabled) {
       ((tcolor_data_obj *) (the_data_rec.item_collection)->at(i))->
	 get_display_str (the_string,max_str_len);
       fprintf (stdprn,"%s\n",the_string);
       }  // end for, if
   } // end fn

// Use these procedures for reading and writing setup records.

void write_record (char *the_file_name,void *the_rec,int the_size)
   {
   int the_file;
   char the_message[52];
   if ((the_file = open (the_file_name,O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
	  S_IREAD | S_IWRITE)) == -1) {
	  strcpy (the_message,"Problem opening file ");
	  strncat (the_message,the_file_name,30);
	  messageBox (the_message,mfError | mfOKButton);
	  return;
	  }
   if (_write (the_file,the_rec,the_size) < 0) {
	  strcpy (the_message,"Problem writing file ");
	  strncat (the_message,the_file_name,30);
	  messageBox (the_message,mfError | mfOKButton);    }
   close (the_file);
   return;
   }

void read_record (char *the_file_name,void *the_rec,int the_size)
   {
   int the_file;
   char the_message[52];
   if ((the_file = open (the_file_name,O_RDONLY | O_BINARY)) == -1) {
      strcpy (the_message,"Problem opening file ");
      strncat (the_message,the_file_name,30);
      messageBox (the_message,mfError | mfOKButton);
      return;
      }
   if (_read (the_file,the_rec,the_size) < 0) {
      strcpy (the_message,"Problem reading file ");
      strncat (the_message,the_file_name,30);
      messageBox (the_message,mfError | mfOKButton); }
   close (the_file);
   return;
   }

#endif
