/* File UTFILES.C  File handling procedures: Open text files, copy file  */

#ifndef UTFILES_C
#define UTFILES_C

#include <stdio.h>
#include <conio.h>
#include <dir.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>

#include "ut.h"

long open_input_text_file (FILE **the_file,path_str file_name);
short open_output_text_file (FILE **the_file,path_str file_name);
int fcopy (const char *source_name,const char *dest_name);

/*  Implementation  */

void file_error_message (path_str file_name,char *in_out) {
     clrscr ();
     gotoxy (1,9);
     cprintf ("Problem opening %s file %s.\r\n",in_out,file_name);
     cprintf ("Hit key to continue: ");
     getch();
     return;
     }

long open_input_text_file (FILE **the_file,path_str file_name) {
   /* Returns file size in bytes if successful, 0 otherwise */
   struct ffblk file_rec;
   if ((!file_name) || (file_name[0] == '\0')) {
      cprintf ("\r\n\nInput file name is blank.");
      getch();
      return 0;
      }
   if (findfirst(file_name,&file_rec,0) == 0) {
      *the_file = fopen(file_name,"rt");
      return file_rec.ff_fsize;
      }
   else file_error_message (file_name,"input");
   return 0;
   }  /* end func */

short open_output_text_file (FILE **the_file,path_str file_name) {
   /* Returns 1 if successful open  */
   if ((!file_name) || (file_name[0] == '\0')) {
      cprintf ("\r\n\nOutput file name is blank.");
      return 0;
      }
   if ((*the_file = fopen(file_name,"wt")) == NULL) {
      file_error_message (file_name,"output");
      return 0;
      }
    return 1;  /*  Successful  */
    }  /*  end func  */

int fcopy (const char *source_name,const char *dest_name) {
    /* General file copying utility */
    /* Returns 0 on successful copy */
    int source_file,dest_file;
    long int file_size,i,no_of_blocks,remainder;
    ldiv_t result;
    char data[200],one_byte;
    long int block_size = sizeof(data);
    if ((source_file = open (source_name,O_RDONLY | O_BINARY)) == -1)
       return -1;
    if ((dest_file = open (dest_name,O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
       S_IREAD | S_IWRITE)) == -1) return -2;
    file_size = filelength (source_file);
    result = ldiv (file_size,block_size);
    no_of_blocks = result.quot;
    remainder = result.rem;
    for (i=1;i<=no_of_blocks;i++) {
	read (source_file,data,block_size);
	write (dest_file,data,block_size);
	}
    for (i=0;i<remainder;i++) {
	read (source_file,&one_byte,1);
	write (dest_file,&one_byte,1);
	}
    close (source_file);
    close (dest_file);
    return 0;
    }

#endif