/*  File UTGRSET.C  Graph setup utility  */
/*  Note: This file uses Borland's graphics.h library, and so is */
/*        not ANSI C compatible.  */

#ifndef UTGRSET_C
#define UTGRSET_C

#include <string.h>
#include <stdio.h>
#include <conio.h>
#include "utgr.h"

const
   /* VGAHi constants */
   vstart_col = 112,
   vend_col = 559,
   vstart_row = 10,
   vend_row = 457,
   vno_of_rows = 448,
   vno_of_cols = 448,
   viter_row_1 = 298,
   viter_row_2 = 320,
   viter_col = 1,
   vdone_row = 362,
   vx_min_col = 112,
   vx_max_col = 488,
   vx_row = 465,
   vy_min_row = 445,
   vy_max_row = 30,
   vy_col = 564,
   vmax_rows = 480,
   vmax_cols = 640;

float gr_x_min,gr_y_min,gr_x_max,gr_y_max,gr_x_range,gr_y_range;
int graphdriver = DETECT,graphmode;
int gr_start_col,gr_end_col,
       gr_start_row,gr_end_row,
       gr_no_of_rows,gr_no_of_cols,
       gr_iter_row_1,gr_iter_row_2,
       gr_iter_col,gr_done_row,
       gr_x_min_col,gr_x_max_col,gr_x_row,
       gr_y_min_row,gr_y_max_row,gr_y_col;

graph_setup_rec gr_graph_setup = {X_MIN_INIT,X_MAX_INIT,Y_MIN_INIT,
       Y_MAX_INIT,MAX_MODE_INIT,SCALE_WINDOW_FROM_DATA_INIT,
       LABEL_AXES_INIT,X_AXIS_LABEL_INIT,Y_AXIS_LABEL_INIT,
       LINE_COLOR_INIT,TEXT_COLOR_INIT,BACK_COLOR_INIT};

short graph_setup (graph_setup_rec the_setup);
void draw_border(graph_setup_rec gr_setup);
void graph_x_y_labels (graph_setup_rec gr_setup);
void graph_iter (graph_setup_rec gr_setup,unsigned long iter);
short xy_to_screen_col_row (float x,float y,int *col,int *row);
void xy_to_screen (float x,float y,int color);

/*  Implementation  */

void convert_gr_constants (void)
   {
   float rmax_rows = getmaxy() + 1;
   float rmax_cols = getmaxx() + 1;
   float col_ratio = rmax_cols/vmax_cols;
   float row_ratio = rmax_rows/vmax_rows;
   gr_start_col = vstart_col * col_ratio;
   gr_end_col = vend_col * col_ratio;
   gr_start_row = vstart_row * row_ratio;
   gr_end_row = vend_row * row_ratio;
   gr_no_of_rows = vno_of_rows * row_ratio;
   gr_no_of_cols = vno_of_cols * col_ratio;
   gr_iter_row_1 = viter_row_1 * row_ratio;
   gr_iter_row_2 = viter_row_2 * row_ratio;
   gr_iter_col = viter_col * col_ratio;
   gr_done_row = vdone_row * row_ratio;
   gr_x_min_col = vx_min_col * col_ratio;
   gr_x_max_col = vx_max_col * col_ratio;
   gr_x_row = vx_row * row_ratio;
   gr_y_min_row = vy_min_row * row_ratio;
   gr_y_max_row = vy_max_row * row_ratio;
   gr_y_col = vy_col * col_ratio;
   return;
   }

short graph_setup (graph_setup_rec the_setup) {
   /* Returns 1 if successful, 0 if not */
   float x_range,y_range,x_midpoint,y_midpoint;
   int errorcode;
   x_range = the_setup.x_max - the_setup.x_min;
   y_range = the_setup.y_max - the_setup.y_min;
   if (the_setup.scale_window_from_data == 0) {
      gr_x_min = the_setup.x_min;
      gr_x_max = the_setup.x_max;
      gr_y_min = the_setup.y_min;
      gr_y_max = the_setup.y_max;
      gr_x_range = gr_x_max - gr_x_min;
      gr_y_range = gr_y_max - gr_y_min;
      }
    else {
      /* Equalize graphic x and y ranges to avoid distortion. */
      if (x_range > y_range) {
	 gr_x_range = 1.1 * x_range;
	 gr_y_range = 1.1 * x_range;
	 }
      else {
	 gr_x_range = 1.1 * y_range;
	 gr_y_range = 1.1 * y_range;
	 }
      y_midpoint = the_setup.y_min + 0.5 * y_range;
      gr_y_min = y_midpoint - 0.5 * gr_y_range;
      gr_y_max = y_midpoint + 0.5 * gr_y_range;
      x_midpoint = the_setup.x_min + 0.5 * x_range;
      gr_x_min = x_midpoint - 0.5 * gr_x_range;
      gr_x_max = x_midpoint + 0.5 * gr_y_range;
      }

    cprintf("\r\n");
    cprintf("Press key to start graphics (esc to cancel): ");
    if (getch() == (char) 27)  return 0;
    initgraph (&graphdriver,&graphmode,"");
    errorcode = graphresult();
    if (errorcode != grOk) {
       cprintf("\nGraphics Error.\n");
       getch();
       return 0;
       }
    if (graphmode > the_setup.max_mode) {
       graphmode = the_setup.max_mode;
       setgraphmode (graphmode);
       }
    convert_gr_constants();
    setbkcolor (the_setup.back_color);
    return 1;
    }

void draw_border(graph_setup_rec gr_setup)
   {
    setcolor (gr_setup.line_color);
    line (gr_start_col,gr_start_row,gr_end_col,gr_start_row);
    line (gr_end_col,gr_start_row,gr_end_col,gr_end_row);
    line (gr_end_col,gr_end_row,gr_start_col,gr_end_row);
    line (gr_start_col,gr_end_row,gr_start_col,gr_start_row);
    return;
   }

void graph_x_y_labels (graph_setup_rec gr_setup)
   {
   char the_str [20];
   setcolor (gr_setup.text_color);
   sprintf (the_str,"%c: %6.3f",gr_setup.x_axis_label,gr_x_min);
   outtextxy (gr_x_min_col,gr_x_row,the_str);
   sprintf (the_str,"%c: %6.3f",gr_setup.x_axis_label,gr_x_max);
   outtextxy (gr_x_max_col,gr_x_row,the_str);
   sprintf (the_str,"%c: %6.3f",gr_setup.y_axis_label,gr_y_max);
   outtextxy (gr_y_col,gr_y_max_row,the_str);
   sprintf (the_str,"%c: %6.3f",gr_setup.y_axis_label,gr_y_min);
   outtextxy (gr_y_col,gr_y_min_row,the_str);
   return;
   }

void graph_iter (graph_setup_rec gr_setup,unsigned long iter)
   /* Clear previous graph iter output and write new iteration value */
   {
     unsigned long previous = iter - 1;
     char clear_str [7], num_str[7];
     sprintf (clear_str,"%6lu",previous);
     setcolor (gr_setup.back_color);
     outtextxy (gr_iter_col,gr_iter_row_2,clear_str);
     sprintf (num_str,"%6lu",iter);
     setcolor (gr_setup.text_color);
     outtextxy (gr_iter_col,gr_iter_row_1,"Iter: ");
     outtextxy (gr_iter_col,gr_iter_row_2,num_str);
     return;
     }

short xy_to_screen_col_row (float x,float y,int *col,int *row)
   {
   /* Returns non_zero value only if point is in valid range */
   short in_range = 1;
   if (x > gr_x_max) return 0;
   if (x < gr_x_min) return 0;
   if (y > gr_y_max) return 0;
   if (y < gr_y_min) return 0;
   *col = gr_start_col + ((x - gr_x_min)/gr_x_range) * gr_no_of_cols;
   *row = gr_end_row - ((y - gr_y_min)/gr_y_range) * gr_no_of_rows;
   return in_range;
   }

void xy_to_screen (float x,float y,int color)
   {
   int scr_color,col,row;
   if (graphmode == CGAHI) scr_color = 1;
   else
      scr_color = color;
   if (xy_to_screen_col_row (x,y,&col,&row) != 0)
      putpixel (col,row,scr_color);
   return;
   }

#endif
