/*  File UTVECT.C  Vector handling routines  */
/*  The memory allocation techniques used here are based on */
/*  the public-domain routines in Appendix B of  */
/*  "Numerical Recipes in C", 2nd Ed., by W. Press, S. Teukolsky, */
/*  W. Vetterling, and B. Flannery, Cambridge University Press (1992) */

#ifndef UTVECT_C
#define UTVECT_C

#include <conio.h>
#include <alloc.h>
#include <mem.h>
#include <stdlib.h>

#include "ut.h"
#define VECT_END 1

ui_vector allocate_ui_vector (int low_limit,int up_limit);
f_vector allocate_f_vector (int low_limit,int up_limit);
d_vector allocate_d_vector (int low_limit,int up_limit);
void copy_ui_vector (ui_vector source,ui_vector dest,int start,int len);
void copy_f_vector (f_vector source,f_vector dest,int start,int len);
void copy_d_vector (d_vector source,d_vector dest,int start,int len);
void free_ui_vector (ui_vector v,int low_limit);
void free_f_vector (f_vector v,int low_limit);
void free_d_vector (d_vector v,int low_limit);

/*  Implementation */

void utvect_error_msg (char *the_msg) {
   textmode (LASTMODE);
   gotoxy (1,5);
   cprintf ("UTVECT.C Run time error: \r\n%s\r\n",the_msg);
   getch();
   exit (1);
   }

ui_vector allocate_ui_vector (int low_limit,int up_limit) {
      ui_vector v;
      int vlen;
      vlen = up_limit - low_limit + 1;
      v = (unsigned int *)malloc ((size_t)(vlen+VECT_END)*sizeof(unsigned int));
      if (!v) utvect_error_msg ("ui_vector allocation failed.");
      return v - low_limit + VECT_END;
      }  /*  end function */

f_vector allocate_f_vector (int low_limit,int up_limit) {
      f_vector v;
      int vlen;
      vlen = up_limit - low_limit + 1;
      v = (float *)malloc ((size_t)(vlen+VECT_END)*sizeof(float));
      if (!v) utvect_error_msg ("f_vector allocation failed.");
      return v - low_limit + VECT_END;
      }  /*  end function */

d_vector allocate_d_vector (int low_limit,int up_limit) {
      d_vector v;
      int vlen;
      vlen = up_limit - low_limit + 1;
      v = (double *)malloc ((size_t)(vlen+VECT_END)*sizeof(double));
      if (!v) utvect_error_msg ("d_vector allocation failed.");
      return v - low_limit + VECT_END;
      }  /*  end function */

void copy_ui_vector (ui_vector source,ui_vector dest,int start,int len) {
     /* Copies len components of source to dest,  */
     /* starting at position start.  */
     memmove (dest + start,source + start,len * sizeof(unsigned int));
     return;
     }  /* end proc  */

void copy_f_vector (f_vector source,f_vector dest,int start,int len) {
     /* Copies len components of source to dest,  */
     /* starting at position start.  */
     memmove (dest + start,source + start,len * sizeof(float));
     return;
     }  /* end proc  */

void copy_d_vector (d_vector source,d_vector dest,int start,int len) {
     /* Copies len components of source to dest,  */
     /* starting at position start.  */
     memmove (dest + start,source + start,len * sizeof(double));
     return;
     }  /* end proc  */

void free_ui_vector (ui_vector v,int low_limit) {
     int heap_value;
     heap_value = heapchecknode(v + low_limit - VECT_END);
     if (heap_value == _USEDENTRY)
	free (v + low_limit - VECT_END);
     else {
	cprintf ("\r\nAttempt to free unallocated or bad ui_vector. Code: %d",
	   heap_value);
	getch();
	}
     return;
     }

void free_f_vector (f_vector v,int low_limit) {
     int heap_value;
     heap_value = heapchecknode(v + low_limit - VECT_END);
     if (heap_value == _USEDENTRY)
	free (v + low_limit - VECT_END);
     else {
	cprintf ("\r\nAttempt to free unallocated or bad f_vector. Code: %d",
	   heap_value);
	getch();
	}
     return;
     }

void free_d_vector (d_vector v,int low_limit) {
     int heap_value;
     heap_value = heapchecknode(v + low_limit - VECT_END);
     if (heap_value == _USEDENTRY)
	free (v + low_limit - VECT_END);
     else {
	cprintf ("\r\nAttempt to free unallocated or bad d_vector. Code: %d",
	   heap_value);
	getch();
	}
     return;
     }

#endif